==> gentoo_0.11.19.orig.tar.gz: gentoo-0.11.19/src/cmd_join.c <==
/*
** 2000-02-12 -	A Join command, to counter the long-existing (um, but still incomplete)
**		Split command. Features neat drag-and-drop support for ordering the files
**		to be joined.
*/

#include "gentoo.h"

#include <fcntl.h>

#include "dialog.h"
#include "dirpane.h"
#include "errors.h"
#include "fileutil.h"
#include "overwrite.h"
#include "progress.h"
#include "sizeutil.h"

#include "cmd_join.h"

#define	CMD_ID	"join"

/* ----------------------------------------------------------------------------------------- */

/* 2000-02-12 -	Do the actual joining of the selected files. Returns TRUE on success. */
static gboolean do_join(MainInfo *min, DirPane *src, DirPane *dst, GtkCList *clist, GtkEntry *entry, gsize total_size)
{
	gchar		buf[PATH_MAX], *str;
	gsize		done_size = 0U;

	/* Protect against empty destination name. */
	if((str = gtk_entry_get_text(entry)) == NULL || *str == '\0')
	{
		err_set(min, EINVAL, CMD_ID, NULL);
		return FALSE;
	}

	ovw_overwrite_begin(min, _("\"%s\" Already Exists - Continue With Join?"), 0U);
	g_snprintf(buf, sizeof buf, "%s/%s", dst->dir.path, str);
	if(ovw_overwrite_file(min, buf, NULL) == OVW_PROCEED)
	{
		gint	fout;

		if((fout = open(buf, O_CREAT | O_WRONLY | O_TRUNC, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) >= 0)
		{
			gint	fin, i;
			DirRow	*dr;
			gsize	put;

			pgs_progress_begin(min, _("Joining..."), PFLG_BYTE_VISIBLE);
			for(i = 0; (dr = gtk_clist_get_row_data(clist, i)) != NULL; i++)
			{
				if((fin = open(dp_full_name(src, DP_ROW_INDEX(src, dr)), O_RDONLY)) >= 0)
				{
					pgs_progress_item_begin(min, DP_ROW_NAME(dr), DP_ROW_STAT(dr).st_size);
					put = fut_copy(fin, fout, 256 << 10);
					pgs_progress_item_end(min);
					close(fin);
					if(put != DP_ROW_STAT(dr).st_size)
					{
						err_set(min, 000, CMD_ID, DP_ROW_NAME(dr));
						break;
					}
					done_size += put;
					dp_unselect(src, DP_ROW_INDEX(src, dr));
				}
				else
				{
					err_set(min, errno, CMD_ID, DP_ROW_NAME(dr));
					break;
				}
			}
			pgs_progress_end(min);
			close(fout);
			if(done_size == total_size)
				dp_rescan(dst);
			else
				unlink(buf);
		}
		else
			err_set(min, errno, CMD_ID, buf);
	}
	ovw_overwrite_end(min);

	return done_size == total_size;
}

int cmd_join(MainInfo *min, DirPane *src, DirPane *dst, CmdArg *ca)
{
	GSList	*sel;
	gint	ret = 1;

	err_clear(min);

	if((sel = dp_get_selection(src)) != NULL)
	{
		GSList	*iter;
		gsize	tot = 0, num = 0;

		/* Compute size of all regular files in the selection. */
		for(iter = sel; iter != NULL; iter = g_slist_next(iter))
		{
			if(S_ISREG(DP_ROW_STAT((DirRow *) iter->data).st_mode))
			{
				tot += DP_ROW_STAT((DirRow *) iter->data).st_size;
				num++;
			}
		}
		
		/* Only open the dialog and proceed with command if there were, in
		** fact, more than one regular file selected.
		*/
		if(num > 1)
		{
			Dialog		*dlg;
			GtkWidget	*scwin, *vbox, *label, *clist, *entry;
			gchar		buf[64];
			gchar		*text[1];
			gint		row;

			vbox = gtk_vbox_new(FALSE, 0);
			label = gtk_label_new(_("Click and Drag Files to Reorder, Then Click Join."));
			gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);

			if(tot > 1 << 10)
			{
				gchar	sbuf[16];

				sze_put_size(tot, sbuf, sizeof sbuf, SZE_AUTO);
				g_snprintf(buf, sizeof buf, _("The total size is %s (%lu bytes)."), sbuf, (unsigned long) tot);
			}
			else
				g_snprintf(buf, sizeof buf, _("The total size is %lu bytes."), (unsigned long) tot);
			label = gtk_label_new(buf);
			gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);

			scwin = gtk_scrolled_window_new(NULL, NULL);
			gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
			clist = gtk_clist_new(1);
			gtk_clist_set_column_auto_resize(GTK_CLIST(clist), 0, TRUE);
			gtk_clist_set_reorderable(GTK_CLIST(clist), TRUE);
			gtk_widget_set_usize(clist, -1, 256);		/* Get a sensible minmum height. */
			for(iter = sel; iter != NULL; iter = g_slist_next(iter))
			{
				if(S_ISREG(DP_ROW_STAT((DirRow *) iter->data).st_mode))
				{
					text[0] = DP_ROW_NAME((DirRow *) iter->data);
					row = gtk_clist_append(GTK_CLIST(clist), text);
					gtk_clist_set_row_data(GTK_CLIST(clist), row, iter->data);
				}
			}
			gtk_container_add(GTK_CONTAINER(scwin), clist);
			gtk_box_pack_start(GTK_BOX(vbox), scwin, TRUE, TRUE, 0);

			label = gtk_label_new(_("Enter Destination File Name"));
			gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 5);
			entry = gtk_entry_new_with_max_length(FILENAME_MAX - 1);
			gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE, 0);

			dlg = dlg_dialog_sync_new(vbox, _("Join"), _("Join|Cancel"));
			gtk_widget_show_all(vbox);
			gtk_widget_grab_focus(entry);
			if(dlg_dialog_sync_wait(dlg) == DLG_POSITIVE)
				ret = do_join(min, src, dst, GTK_CLIST(clist), GTK_ENTRY(entry), tot);
			dlg_dialog_sync_destroy(dlg);
		}
		dp_free_selection(sel);
	}
	err_show(min);
	return ret;
}

==> gentoo-0.11.34.tar.gz: gentoo-0.11.34/src/cmd_join.c <==
/*
** 2000-02-12 -	A Join command, to counter the long-existing (um, but still incomplete)
**		Split command. Features neat drag-and-drop support for ordering the files
**		to be joined.
*/

#include "gentoo.h"

#include <fcntl.h>

#include "dialog.h"
#include "dirpane.h"
#include "errors.h"
#include "fileutil.h"
#include "overwrite.h"
#include "progress.h"
#include "sizeutil.h"

#include "cmd_join.h"

#define	CMD_ID	"join"

/* ----------------------------------------------------------------------------------------- */

/* 2000-02-12 -	Do the actual joining of the selected files. Returns TRUE on success. */
static gboolean do_join(MainInfo *min, DirPane *src, DirPane *dst, GtkCList *clist, GtkEntry *entry, gsize total_size)
{
	gchar		buf[PATH_MAX], *str;
	gsize		done_size = 0U;

	/* Protect against empty destination name. */
	if((str = gtk_entry_get_text(entry)) == NULL || *str == '\0')
	{
		err_set(min, EINVAL, CMD_ID, NULL);
		return FALSE;
	}

	ovw_overwrite_begin(min, _("\"%s\" Already Exists - Continue With Join?"), 0U);
	g_snprintf(buf, sizeof buf, "%s/%s", dst->dir.path, str);
	if(ovw_overwrite_file(min, buf, NULL) == OVW_PROCEED)
	{
		gint	fout;

		if((fout = open(buf, O_CREAT | O_WRONLY | O_TRUNC, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) >= 0)
		{
			gint	fin, i;
			DirRow	*dr;
			gsize	put;

			pgs_progress_begin(min, _("Joining..."), PFLG_BYTE_VISIBLE);
			for(i = 0; (dr = gtk_clist_get_row_data(clist, i)) != NULL; i++)
			{
				if((fin = open(dp_full_name(src, DP_ROW_INDEX(src, dr)), O_RDONLY)) >= 0)
				{
					pgs_progress_item_begin(min, DP_ROW_NAME(dr), DP_ROW_STAT(dr).st_size);
					put = fut_copy(fin, fout, 256 << 10);
					pgs_progress_item_end(min);
					close(fin);
					if(put != DP_ROW_STAT(dr).st_size)
					{
						err_set(min, 000, CMD_ID, DP_ROW_NAME(dr));
						break;
					}
					done_size += put;
					dp_unselect(src, DP_ROW_INDEX(src, dr));
				}
				else
				{
					err_set(min, errno, CMD_ID, DP_ROW_NAME(dr));
					break;
				}
			}
			pgs_progress_end(min);
			close(fout);
			if(done_size == total_size)
				dp_rescan_post_cmd(dst);
			else
				unlink(buf);
		}
		else
			err_set(min, errno, CMD_ID, buf);
	}
	ovw_overwrite_end(min);

	return done_size == total_size;
}

gint cmd_join(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
{
	GSList	*sel;
	gint	ret = 1;

	err_clear(min);

	if((sel = dp_get_selection(src)) != NULL)
	{
		GSList	*iter;
		gsize	tot = 0, num = 0;

		/* Compute size of all regular files in the selection. */
		for(iter = sel; iter != NULL; iter = g_slist_next(iter))
		{
			if(S_ISREG(DP_ROW_STAT((DirRow *) iter->data).st_mode))
			{
				tot += DP_ROW_STAT((DirRow *) iter->data).st_size;
				num++;
			}
		}
		
		/* Only open the dialog and proceed with command if there were, in
		** fact, more than one regular file selected.
		*/
		if(num > 1)
		{
			Dialog		*dlg;
			GtkWidget	*scwin, *vbox, *label, *clist, *entry;
			gchar		buf[64];
			const gchar	*text[1];
			gint		row;

			vbox  = gtk_vbox_new(FALSE, 0);
			label = gtk_label_new(_("Click and Drag Files to Reorder, Then Click Join."));
			gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);

			if(tot > 1 << 10)
			{
				gchar	sbuf[16];

				sze_put_size(sbuf, sizeof sbuf, tot, SZE_AUTO);
				g_snprintf(buf, sizeof buf, _("The total size is %s (%lu bytes)."), sbuf, (unsigned long) tot);
			}
			else
				g_snprintf(buf, sizeof buf, _("The total size is %lu bytes."), (unsigned long) tot);
			label = gtk_label_new(buf);
			gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);

			scwin = gtk_scrolled_window_new(NULL, NULL);
			gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
			clist = gtk_clist_new(1);
			gtk_clist_set_column_auto_resize(GTK_CLIST(clist), 0, TRUE);
			gtk_clist_set_reorderable(GTK_CLIST(clist), TRUE);
			gtk_widget_set_usize(clist, -1, 256);		/* Get a sensible minmum height. */
			for(iter = sel; iter != NULL; iter = g_slist_next(iter))
			{
				if(S_ISREG(DP_ROW_STAT((DirRow *) iter->data).st_mode))
				{
					text[0] = DP_ROW_NAME((DirRow *) iter->data);
					row = gtk_clist_append(GTK_CLIST(clist), (gchar **) text);
					gtk_clist_set_row_data(GTK_CLIST(clist), row, iter->data);
				}
			}
			gtk_container_add(GTK_CONTAINER(scwin), clist);
			gtk_box_pack_start(GTK_BOX(vbox), scwin, TRUE, TRUE, 0);

			label = gtk_label_new(_("Enter Destination File Name"));
			gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 5);
			entry = gtk_entry_new_with_max_length(FILENAME_MAX - 1);
			gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE, 0);

			dlg = dlg_dialog_sync_new(vbox, _("Join"), _("_Join|_Cancel"));
			gtk_widget_show_all(vbox);
			gtk_widget_grab_focus(entry);
			if(dlg_dialog_sync_wait(dlg) == DLG_POSITIVE)
				ret = do_join(min, src, dst, GTK_CLIST(clist), GTK_ENTRY(entry), tot);
			dlg_dialog_sync_destroy(dlg);
		}
		dp_free_selection(sel);
	}
	err_show(min);
	return ret;
}
